@@ -20,6 +20,7 @@ import subprocess
 import tempfile
 
 import fixtures
+from keystoneauth1 import exceptions as ks_exceptions
 import mock
 from mistralclient.api import base as mistralclient_base
 from novaclient import exceptions
@@ -53,10 +54,9 @@ class TestUndercloud(BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._run_instack')
     @mock.patch('instack_undercloud.undercloud._generate_environment')
     @mock.patch('instack_undercloud.undercloud._load_config')
-    @mock.patch('instack_undercloud.undercloud._die_tuskar_die')
-    def test_install(self, mock_die_tuskar_die, mock_load_config,
-                     mock_generate_environment, mock_run_instack,
-                     mock_run_clean_all, mock_run_yum_update, mock_run_orc,
+    def test_install(self, mock_load_config, mock_generate_environment,
+                     mock_run_instack, mock_run_clean_all,
+                     mock_run_yum_update, mock_run_orc,
                      mock_post_config, mock_run_command,
                      mock_validate_configuration, mock_configure_logging,
                      mock_upgrade_fact):
@@ -70,7 +70,6 @@ class TestUndercloud(BaseTestCase):
         mock_run_command.assert_called_with(
             ['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
         mock_upgrade_fact.assert_called_with(False)
-        mock_die_tuskar_die.assert_not_called()
 
     @mock.patch('instack_undercloud.undercloud._handle_upgrade_fact')
     @mock.patch('instack_undercloud.undercloud._configure_logging')
@@ -83,11 +82,10 @@ class TestUndercloud(BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._run_instack')
     @mock.patch('instack_undercloud.undercloud._generate_environment')
     @mock.patch('instack_undercloud.undercloud._load_config')
-    @mock.patch('instack_undercloud.undercloud._die_tuskar_die')
-    def test_install_upgrade(self, mock_die_tuskar_die, mock_load_config,
-                             mock_generate_environment, mock_run_instack,
-                             mock_run_yum_clean_all, mock_run_yum_update,
-                             mock_run_orc, mock_post_config, mock_run_command,
+    def test_install_upgrade(self, mock_load_config, mock_generate_environment,
+                             mock_run_instack, mock_run_yum_clean_all,
+                             mock_run_yum_update, mock_run_orc,
+                             mock_post_config, mock_run_command,
                              mock_validate_configuration,
                              mock_configure_logging, mock_upgrade_fact):
         fake_env = mock.MagicMock()
@@ -100,7 +98,6 @@ class TestUndercloud(BaseTestCase):
         mock_run_command.assert_called_with(
             ['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
         mock_upgrade_fact.assert_called_with(True)
-        mock_die_tuskar_die.assert_called_once()
 
     @mock.patch('instack_undercloud.undercloud._configure_logging')
     def test_install_exception(self, mock_configure_logging):
@@ -149,13 +146,10 @@ class TestUndercloud(BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._check_sysctl')
     @mock.patch('instack_undercloud.undercloud._validate_network')
     @mock.patch('instack_undercloud.undercloud._validate_no_ip_change')
-    @mock.patch('instack_undercloud.undercloud._validate_passwords_file')
-    def test_validate_configuration(self, mock_vpf, mock_vnic,
-                                    mock_validate_network,
+    def test_validate_configuration(self, mock_vnic, mock_validate_network,
                                     mock_check_memory, mock_check_hostname,
                                     mock_check_sysctl):
         undercloud._validate_configuration()
-        self.assertTrue(mock_vpf.called)
         self.assertTrue(mock_vnic.called)
         self.assertTrue(mock_validate_network.called)
         self.assertTrue(mock_check_memory.called)
@@ -246,18 +240,33 @@ class TestCheckHostname(BaseTestCase):
 
 
 class TestCheckMemory(BaseTestCase):
+    @mock.patch('psutil.swap_memory')
     @mock.patch('psutil.virtual_memory')
-    def test_sufficient_memory(self, mock_vm):
+    def test_sufficient_memory(self, mock_vm, mock_sm):
         mock_vm.return_value = mock.Mock()
         mock_vm.return_value.total = 8589934592
+        mock_sm.return_value = mock.Mock()
+        mock_sm.return_value.total = 0
         undercloud._check_memory()
 
+    @mock.patch('psutil.swap_memory')
     @mock.patch('psutil.virtual_memory')
-    def test_insufficient_memory(self, mock_vm):
+    def test_insufficient_memory(self, mock_vm, mock_sm):
         mock_vm.return_value = mock.Mock()
         mock_vm.return_value.total = 2071963648
+        mock_sm.return_value = mock.Mock()
+        mock_sm.return_value.total = 0
         self.assertRaises(RuntimeError, undercloud._check_memory)
 
+    @mock.patch('psutil.swap_memory')
+    @mock.patch('psutil.virtual_memory')
+    def test_sufficient_swap(self, mock_vm, mock_sm):
+        mock_vm.return_value = mock.Mock()
+        mock_vm.return_value.total = 6442450944
+        mock_sm.return_value = mock.Mock()
+        mock_sm.return_value.total = 2147483648
+        undercloud._check_memory()
+
 
 class TestCheckSysctl(BaseTestCase):
     @mock.patch('os.path.isfile')
@@ -313,22 +322,6 @@ class TestNoIPChange(BaseTestCase):
         undercloud._validate_no_ip_change()
 
 
-@mock.patch('os.path.isfile')
-class TestPasswordsFileExists(BaseTestCase):
-    def test_new_install(self, mock_isfile):
-        mock_isfile.side_effect = [False]
-        undercloud._validate_passwords_file()
-
-    def test_update_exists(self, mock_isfile):
-        mock_isfile.side_effect = [True, True]
-        undercloud._validate_passwords_file()
-
-    def test_update_missing(self, mock_isfile):
-        mock_isfile.side_effect = [True, False]
-        self.assertRaises(validator.FailedValidation,
-                          undercloud._validate_passwords_file)
-
-
 class TestGenerateEnvironment(BaseTestCase):
     def setUp(self):
         super(TestGenerateEnvironment, self).setUp()
@@ -347,11 +340,6 @@ class TestGenerateEnvironment(BaseTestCase):
         self.useFixture(self.mock_distro)
         self.mock_distro.mock.return_value = [
             'Red Hat Enterprise Linux Server 7.1']
-        stackrc_exists_patcher = mock.patch(
-            'instack_undercloud.undercloud._stackrc_exists',
-            return_value=False)
-        stackrc_exists_patcher.start()
-        self.addCleanup(stackrc_exists_patcher.stop)
 
     @mock.patch('socket.gethostname')
     def test_hostname_set(self, mock_gethostname):
@@ -414,7 +402,7 @@ class TestGenerateEnvironment(BaseTestCase):
         # Instead make sure that it looks like a valid list by parsing it.
         drivers = json.loads(env['ENABLED_DRIVERS'])
         self.assertEqual(sorted(drivers), ['pxe_drac', 'pxe_ilo',
-                                           'pxe_ipmitool', 'pxe_ssh'])
+                                           'pxe_ipmitool'])
         self.assertEqual(env['INSPECTION_NODE_NOT_FOUND_HOOK'], '')
 
     def test_enabled_discovery(self):
@@ -428,7 +416,7 @@ class TestGenerateEnvironment(BaseTestCase):
         drivers = json.loads(env['ENABLED_DRIVERS'])
         # Discovery requires enabling the default driver
         self.assertEqual(sorted(drivers), ['foobar', 'pxe_drac', 'pxe_ilo',
-                                           'pxe_ipmitool', 'pxe_ssh'])
+                                           'pxe_ipmitool'])
         self.assertEqual(env['INSPECTION_NODE_NOT_FOUND_HOOK'], 'enroll')
 
     def test_generate_endpoints(self):
@@ -645,6 +633,7 @@ class TestConfigureSshKeys(base.BaseTestCase):
 
 
 class TestPostConfig(base.BaseTestCase):
+    @mock.patch('instack_undercloud.undercloud._member_role_exists')
     @mock.patch('novaclient.client.Client', autospec=True)
     @mock.patch('mistralclient.api.client.client', autospec=True)
     @mock.patch('instack_undercloud.undercloud._delete_default_flavors')
@@ -656,7 +645,7 @@ class TestPostConfig(base.BaseTestCase):
     def test_post_config(self, mock_post_config_mistral, mock_ensure_flavor,
                          mock_configure_ssh_keys, mock_get_auth_values,
                          mock_copy_stackrc, mock_delete, mock_mistral_client,
-                         mock_nova_client):
+                         mock_nova_client, mock_member_role_exists):
         instack_env = {
             'UNDERCLOUD_ENDPOINT_MISTRAL_PUBLIC':
                 'http://192.168.24.1:8989/v2',
@@ -714,45 +703,26 @@ class TestPostConfig(base.BaseTestCase):
             mistralclient_base.APIException)
 
         env = {
-            "UNDERCLOUD_DB_PASSWORD": "root-db-pass",
             "UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD": "snmpd-pass"
         }
+        json_string = '{"undercloud_ceilometer_snmpd_password": "snmpd-pass"}'
 
-        json_string = {
-            "undercloud_db_password": "root-db-pass",
-            "undercloud_ceilometer_snmpd_password": "snmpd-pass"
-        }
-
-        undercloud._create_mistral_config_environment(json.loads(
-            json.dumps(env, sort_keys=True)), mock_mistral)
+        undercloud._create_mistral_config_environment(env, mock_mistral)
 
         mock_mistral.environments.create.assert_called_once_with(
-            name='tripleo.undercloud-config',
-            description='Undercloud configuration parameters',
-            variables=json.dumps(json_string, sort_keys=True))
+            name="tripleo.undercloud-config",
+            variables=json_string)
 
     def test_create_config_environment_existing(self):
         mock_mistral = mock.Mock()
-        environment = collections.namedtuple('environment',
-                                             ['name', 'variables'])
-
-        json_string = {
-            "undercloud_db_password": "root-db-pass",
-            "undercloud_ceilometer_snmpd_password": "snmpd-pass"
-        }
-
+        environment = collections.namedtuple('environment', ['name'])
         mock_mistral.environments.get.return_value = environment(
-            name='tripleo.undercloud-config',
-            variables=json.loads(json.dumps(json_string, sort_keys=True))
-           )
-
+            name='overcloud')
         env = {
-            "UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD": "snmpd-pass",
-            "UNDERCLOUD_DB_PASSWORD": "root-db-pass"
+            "UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD": "snmpd-pass"
         }
 
-        undercloud._create_mistral_config_environment(json.loads(
-            json.dumps(env, sort_keys=True)), mock_mistral)
+        undercloud._create_mistral_config_environment(env, mock_mistral)
         mock_mistral.executions.create.assert_not_called()
 
     def test_prepare_ssh_environment(self):
@@ -790,13 +760,6 @@ class TestPostConfig(base.BaseTestCase):
                  ]
         mock_run.assert_has_calls(calls)
 
-    @mock.patch('instack_undercloud.undercloud._stackrc_exists')
-    def test_member_role_exists_nostackrc(self, mock_stackrc_exists):
-        mock_stackrc_exists.return_value = False
-        instack_env = {}
-        undercloud._member_role_exists(instack_env)
-        self.assertEqual({'MEMBER_ROLE_EXISTS': 'False'}, instack_env)
-
     def _mock_ksclient_roles(self, mock_auth_values, mock_ksdiscover, roles):
         mock_auth_values.return_value = ('user', 'password',
                                          'tenant', 'http://test:123')
@@ -813,29 +776,62 @@ class TestPostConfig(base.BaseTestCase):
         mock_client.roles = mock_roles
         mock_discover.create_client.return_value = mock_client
 
-    @mock.patch('keystoneclient.discover.Discover')
-    @mock.patch('instack_undercloud.undercloud._get_auth_values')
-    @mock.patch('instack_undercloud.undercloud._stackrc_exists')
-    def test_member_role_exists(self, mock_stackrc_exists, mock_auth_values,
-                                mock_ksdiscover):
-        mock_stackrc_exists.return_value = True
-        self._mock_ksclient_roles(mock_auth_values, mock_ksdiscover,
-                                  ['admin'])
-        instack_env = {}
-        undercloud._member_role_exists(instack_env)
-        self.assertEqual({'MEMBER_ROLE_EXISTS': 'False'}, instack_env)
+        mock_tenant_list = [mock.Mock(), mock.Mock()]
+        mock_tenant_list[0].name = 'admin'
+        mock_tenant_list[0].id = 'admin-id'
+        mock_tenant_list[1].name = 'service'
+        mock_tenant_list[1].id = 'service-id'
+        mock_client.tenants.list.return_value = mock_tenant_list
+
+        mock_user_list = [mock.Mock(), mock.Mock()]
+        mock_user_list[0].name = 'admin'
+        mock_user_list[1].name = 'nova'
+        mock_client.users.list.return_value = mock_user_list
+        return mock_client
 
     @mock.patch('keystoneclient.discover.Discover')
     @mock.patch('instack_undercloud.undercloud._get_auth_values')
-    @mock.patch('instack_undercloud.undercloud._stackrc_exists')
-    def test_member_role_exists_true(self, mock_stackrc_exists,
+    @mock.patch('os.path.isfile')
+    def test_member_role_exists(self, mock_isfile, mock_auth_values,
+                                mock_ksdiscover):
+        mock_isfile.return_value = True
+        mock_client = self._mock_ksclient_roles(mock_auth_values,
+                                                mock_ksdiscover,
+                                                ['admin'])
+        undercloud._member_role_exists()
+        self.assertFalse(mock_client.tenants.list.called)
+
+    @mock.patch('keystoneclient.discover.Discover')
+    @mock.patch('instack_undercloud.undercloud._get_auth_values')
+    @mock.patch('os.path.isfile')
+    def test_member_role_exists_true(self, mock_isfile,
                                      mock_auth_values, mock_ksdiscover):
-        mock_stackrc_exists.return_value = True
-        self._mock_ksclient_roles(mock_auth_values, mock_ksdiscover,
-                                  ['admin', '_member_'])
-        instack_env = {}
-        undercloud._member_role_exists(instack_env)
-        self.assertEqual({'MEMBER_ROLE_EXISTS': 'True'}, instack_env)
+        mock_isfile.return_value = True
+        mock_client = self._mock_ksclient_roles(mock_auth_values,
+                                                mock_ksdiscover,
+                                                ['admin', '_member_'])
+        undercloud._member_role_exists()
+        mock_user = mock_client.users.list.return_value[0]
+        mock_role = mock_client.roles.list.return_value[1]
+        mock_client.roles.add_user_role.assert_called_once_with(
+            mock_user, mock_role, 'admin-id')
+
+    @mock.patch('keystoneclient.discover.Discover')
+    @mock.patch('instack_undercloud.undercloud._get_auth_values')
+    @mock.patch('os.path.isfile')
+    def test_has_member_role(self, mock_isfile, mock_auth_values,
+                             mock_ksdiscover):
+        mock_isfile.return_value = True
+        mock_client = self._mock_ksclient_roles(mock_auth_values,
+                                                mock_ksdiscover,
+                                                ['admin', '_member_'])
+        fake_exception = ks_exceptions.http.Conflict('test')
+        mock_client.roles.add_user_role.side_effect = fake_exception
+        undercloud._member_role_exists()
+        mock_user = mock_client.users.list.return_value[0]
+        mock_role = mock_client.roles.list.return_value[1]
+        mock_client.roles.add_user_role.assert_called_once_with(
+            mock_user, mock_role, 'admin-id')
 
     def _create_flavor_mocks(self):
         mock_nova = mock.Mock()
